@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------------
 # core.py - basic audio class
 # -----------------------------------------------------------------------------
-# $Id: core.py 2818 2007-09-16 10:04:15Z dmeyer $
+# $Id: core.py 3647 2008-10-25 19:52:16Z hmeine $
 #
 # -----------------------------------------------------------------------------
 # kaa-Metadata - Media Metadata for Python
@@ -29,6 +29,10 @@
 #
 # -----------------------------------------------------------------------------
 
+# python imports
+import sys
+
+# kaa imports
 from kaa.metadata.core import ParseError, Media, MEDIA_AUDIO
 
 AUDIOCORE = ['channels', 'samplerate', 'length', 'encoder', 'codec', 'format',
