@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------------
 # mkv.py - Matroska Streaming Video Files
 # -----------------------------------------------------------------------------
-# $Id: mkv.py 2828 2007-09-17 14:34:21Z tack $
+# $Id: mkv.py 3647 2008-10-25 19:52:16Z hmeine $
 #
 # -----------------------------------------------------------------------------
 # kaa-Metadata - Media Metadata for Python
@@ -31,14 +31,7 @@
 __all__ = ['Parser']
 
 # python imports
-import struct
-import re
-import stat
-import os
-import math
-from types import *
-from struct import *
-from string import *
+from struct import unpack
 import logging
 
 # import kaa.metadata.video core
@@ -280,7 +273,8 @@ class EbmlEntity:
 
 class Matroska(core.AVContainer):
     """
-    This is the main Matroska object
+    Matroska video and audio parser. If at least one video stream is
+    detected it will set the type to MEDIA_AV.
     """
     media = core.MEDIA_AUDIO
 
@@ -361,7 +355,7 @@ class Matroska(core.AVContainer):
 
         elif elem_id == MATROSKA_CUES_ID:
             self.has_idx = True
-            
+
         log.debug('END: process element %s' % hex(elem_id))
         return True
 
@@ -431,7 +425,6 @@ class Matroska(core.AVContainer):
         elif track_type == MATROSKA_SUBTITLES_TRACK:
             log.debug("Subtitle track found")
             track = core.Subtitle()
-            track.language = u'und'
             track.id = len(self.subtitles)
             self.subtitles.append(track)
             for elem in elements:
@@ -441,8 +434,10 @@ class Matroska(core.AVContainer):
     def process_track_common(self, elem, track):
         elem_id = elem.get_id()
         if elem_id == MATROSKA_TRACK_LANGUAGE_ID:
-            track.language = elem.get_str()
-            log.debug("Track language found: %s" % track.language)
+            lang = elem.get_str()
+            if lang != u'und':
+                track.language = lang
+                log.debug("Track language found: %s" % track.language)
         elif elem_id == MATROSKA_NAME_ID:
             track.title = elem.get_utf8()
         elif elem_id == MATROSKA_TRACK_NUMBER_ID:
@@ -455,7 +450,6 @@ class Matroska(core.AVContainer):
         # Defaults
         track.codec = u'Unknown'
         track.fps = 0
-        track.language = u'und'
 
         for elem in elements:
             elem_id = elem.get_id()
@@ -516,7 +510,6 @@ class Matroska(core.AVContainer):
     def process_audio_track(self, elements):
         track = core.AudioStream()
         track.codec = u'Unknown'
-        track.language = u'und'
 
         for elem in elements:
             elem_id = elem.get_id()
