@@ -2,7 +2,7 @@
 # -----------------------------------------------------------------------------
 # mpeg.py - mpeg 1/2 file parser
 # -----------------------------------------------------------------------------
-# $Id: mpeg.py 2581 2007-03-22 14:16:50Z tack $
+# $Id: mpeg.py 3647 2008-10-25 19:52:16Z hmeine $
 #
 # -----------------------------------------------------------------------------
 # kaa-Metadata - Media Metadata for Python
@@ -32,10 +32,8 @@
 __all__ = ['Parser']
 
 # python imports
-import re
 import os
 import struct
-import string
 import logging
 import stat
 
@@ -136,6 +134,14 @@ ASPECT_RATIO = ( None,     # Forbidden
 
 
 class MPEG(core.AVContainer):
+    """
+    Parser for various MPEG files. This includes MPEG-1 and MPEG-2
+    program streams, elementary streams and transport streams. The
+    reported length differs from the length reported by most video
+    players but the provides length here is correct. An MPEG file has
+    no additional metadata like title, etc; only codecs, length and
+    resolution is reported back.
+    """
     def __init__(self,file):
         core.AVContainer.__init__(self)
         self.sequence_header_offset = 0
@@ -147,8 +153,23 @@ class MPEG(core.AVContainer):
             if not self.isMPEG(file):
                 # detect PES
                 if not self.isPES(file):
-                    # no mpeg at all
-                    raise core.ParseError()
+                    # Maybe it's MPEG-ES
+                    if self.isES(file):
+                        # If isES() succeeds, we needn't do anything further.
+                        return
+                    if file.name.lower().endswith('mpeg') or \
+                             file.name.lower().endswith('mpg'):
+                        # This has to be an mpeg file. It could be a bad
+                        # recording from an ivtv based hardware encoder with
+                        # same bytes missing at the beginning.
+                        # Do some more digging...
+                        if not self.isMPEG(file, force=True) or \
+                           not self.video or not self.audio:
+                            # does not look like an mpeg at all
+                            raise core.ParseError()
+                    else:
+                        # no mpeg at all
+                        raise core.ParseError()
 
         self.mime = 'video/mpeg'
         if not self.video:
@@ -423,7 +444,7 @@ class MPEG(core.AVContainer):
 
     # Normal MPEG (VCD, SVCD) ========================================
 
-    def isMPEG(self, file):
+    def isMPEG(self, file, force=False):
         """
         This MPEG starts with a sequence of 0x00 followed by a PACK Header
         http://dvd.sourceforge.net/dvdinfo/packhdr.html
@@ -438,8 +459,15 @@ class MPEG(core.AVContainer):
         offset -= 2
 
         # test for mpeg header 0x00 0x00 0x01
-        if not buffer[offset:offset+4] == '\x00\x00\x01%s' % chr(PACK_PKT):
-            return 0
+        header = '\x00\x00\x01%s' % chr(PACK_PKT)
+        if offset < 0 or not buffer[offset:offset+4] == header:
+            if not force:
+                return 0
+            # brute force and try to find the pack header in the first
+            # 10000 bytes somehow
+            offset = buffer.find(header)
+            if offset < 0:
+                return 0
 
         # scan the 100000 bytes of data
         buffer += file.read(100000)
@@ -651,6 +679,33 @@ class MPEG(core.AVContainer):
         return None
 
 
+    # Elementary Stream ===============================================
+
+    def isES(self, file):
+        file.seek(0, 0)
+        try:
+            header = struct.unpack('>LL', file.read(8))
+        except:
+            return False
+
+        if header[0] != 0x1B3:
+            return False
+
+        # Is an mpeg video elementary stream
+
+        self.mime = 'video/mpeg'
+        video = core.VideoStream()
+        video.width = header[1] >> 20
+        video.height = (header[1] >> 8) & 0xfff
+        if header[1] & 0xf < len(FRAME_RATE):
+            video.fps = FRAME_RATE[header[1] & 0xf]
+        if (header[1] >> 4) & 0xf < len(ASPECT_RATIO):
+            # FIXME: Empirically the aspect looks like PAR rather than DAR
+            video.aspect = ASPECT_RATIO[(header[1] >> 4) & 0xf]
+        self.video.append(video)
+        return True
+
+
     # Transport Stream ===============================================
 
     def isTS(self, file):
